
(**********************************************************************)
(*                                                                    *)
(*                       ASCII to PASCAL v1.50                        *)
(*                                                                    *)
(*             This program converts an ASCII text file,              *)
(*             to a Turbo Pascal program format.                      *)
(*                                                                    *)
(*                   GOTOXY(x_Axis, y_Axis);                          *)
(*                   WRITE( string );                                 *)
(*                                                                    *)
(*    NOTE: Character positions are maintained in conversion.         *)
(*                                                                    *)
(*             Last modification: December 27, 1990                   *)
(*             Written by Guy McLoughlin.                             *)
(*                                                                    *)
(**********************************************************************)

program A2P;

  {$A+,B-,D-,E-,F-,I+,L-,N-,O-,R-,S-,V+}

uses
  crt, dos;

const
  ScreenWidth = 80;
  ScreenLength = 25;

type
  FileNameType = string[14];
  ColorString  = string[12];

  {$I ENTRFLNM.INC}                    (* Filename-entry include file.        *)

const
  ColorSet1 : array[0..7] of ColorString = ('black', 'blue', 'green',
                                            'cyan', 'red', 'magenta',
                                            'brown', 'lightgray');

  ColorSet2 : array[8..15] of ColorString = ('darkgray', 'lightblue',
                                             'lightgreen', 'lightcyan',
                                             'lightred', 'lightmagenta',
                                             'yellow', 'white');

var
  ColorMode,
  DefaultBackColor,
  DefaultTextColor : boolean;

  Index,                               (* Generic loop control variable.      *)
  SampleTextAttr,                      (* Attribute for sample text.          *)
  SampleBackAttr   : byte;             (* Attribute for sample background.    *)

  FirstPartFN      : string[8];        (* First 8 characters of filename.     *)

  TextAttr,                            (* String name for text color.         *)
  BackAttr         : ColorString;      (* String name for background color.   *)

  InFileName,                          (* Filename passed to "A2TP" program.  *)
  OutFileName      : FileNameType;     (* FileName of new ".TP" FILE.         *)

  InString         : string[ScreenWidth];

  InFile,
  OutFile          : text;


  procedure CheckColor(var Color : boolean); (* Check if video is in color    *)
  begin                                      (* mode.                         *)
    if ((mem[0000:1040] and 48) <> 48) then
      Color := true
    else
      Color := false
  end;


  function DnCase(InChar : char) : char;  (* Convert character to lowercase.  *)
  begin
    if ('A' <= InChar) and (InChar <= 'Z') then
      InChar := chr(ord(InChar) + 32);
    DnCase := InChar
  end;


  function BackColorOk : boolean;      (* Check if the TextBackgroundColor    *)
  var                                  (* attribute is a member of ColorSet1. *)
    result : boolean;

  begin
    result := false;
    for Index := 0 to 7 do
      if BackAttr = ColorSet1[Index] then
        begin
          SampleBackAttr := Index;
          result := true
        end;
    BackColorOk := result
  end;


  procedure DefaultBack;               (* Set default the TextBackgroundColor *)
  begin                                (* attribute.                          *)
    DefaultBackColor := true;
    if ColorMode then                  (* If ColorMode = true, then...        *)
      begin
        BackAttr := 'blue';
        SampleBackAttr := 1
      end
    else                               (* else if ColorMode = false, then...  *)
      begin
        BackAttr := 'black';
        SampleBackAttr := 0
      end
  end;


  procedure DefaultText;               (* Set the default TextColor attribute *)
  begin
    DefaultTextColor := true;
    if ColorMode then                  (* If ColorMode = true, then...        *)
      begin
        TextAttr := 'white';
        SampleTextAttr := 15
      end
    else                               (* Else if ColorMode = false, then...  *)
      begin
        TextAttr := 'lightgray';
        SampleTextAttr := 7
      end
  end;


  function TextColorOk : boolean;      (* Check if the TextColor attribute    *)
  var                                  (* is a member of ColorSet2.           *)
    result : boolean;

  begin
    result := false;
    for Index := 0 to 7 do
      if TextAttr = ColorSet1[Index] then
        begin
          SampleTextAttr := Index;
          result := true
        end;
    if result = false then
      for Index := 8 to 15 do
        if TextAttr = ColorSet2[Index] then
          begin
            SampleTextAttr := Index;
            result := true
          end;
    if TextAttr = BackAttr then        (* If Text and background are the same *)
      begin                            (* color attribute.                    *)
        DefaultText;
        DefaultBack
      end;
    TextColorOk := result
  end;


procedure Syntax;                      (* Display program syntax.             *)
begin
  DefaultBack;
  DefaultText;
  gotoxy( 4, 3);
  writeln('Syntax: A2P <FileName> <TextColor> <TextBackgroundColor>');
  gotoxy( 8, 5);
  writeln('ie: A2P                     - This will prompt for the filename to');
  gotoxy(38, 6);
  writeln('convert, then create a file with the');
  gotoxy(38, 7);
  writeln('same name; plus the extension ".A2P",');
  gotoxy(38, 8);
  writeln('with default colors.');
  gotoxy( 8,10);
  writeln('ie: A2P NEWFILE             - This creates "NEWFILE.A2P", with default');
  gotoxy(38,11);
  writeln('colors.');
  gotoxy( 8,14);
  writeln('ie: A2P NEWFILE RED         - This creates "NEWFILE.A2P", with red text,');
  gotoxy(38,15);
  writeln('and default TextBackgroundColor.');
  gotoxy( 8,18);
  writeln('ie: A2P NEWFILE RED GREEN   - This creates "NEWFILE.A2P", with red text,');
  gotoxy(38,19);
  writeln('and green TextBackgroundColor.');
  halt
end;


  procedure DisplaySampleText;         (* Display a sample of how text will   *)
  begin                                (* appear.                             *)
    if DefaultTextColor then
      write('        DEFAULT Text Color  : ')
    else
      write('                Text Color  : ');
    writeln(TextAttr);
    if DefaultBackColor then
      write('  DEFAULT Background Color  : ')
    else
      write('          Background Color  : ');
    writeln(BackAttr);
    writeln;
    write('  ');
    textcolor(SampleTextAttr);
    textbackground(SampleBackAttr);
    gotoxy(14,7);
    writeln('Sample of Text Colors.');
    normvideo;
    writeln
  end;


  procedure NameOutFile(InFileName : FileNameType;    (* Create new filename. *)
                        var OutFileName : FileNameType);
  var
    Index,
    DotMarker : byte;                  (* Used to indicate position of period. *)

  begin
    DotMarker := pos('.', InFileName);
    if DotMarker = 0 then
      FirstPartFN := InFileName
    else
      FirstPartFN := copy(InFileName, 1, (DotMarker - 1));
    OutFileName := concat(FirstPartFN, '.A2P')
  end;


  procedure ProcessParams;             (* Process command-line parameters.    *)
  begin
    if paramstr(1) = '?' then          (* Test if Syntax required.            *)
      Syntax;
    InFileName := '';
    DefaultBackColor := false;
    DefaultTextColor := false;
    writeln;
    writeln('  New file with ''A2P'' extension will be created.');
    writeln;
    if paramcount < 3 then
      DefaultBack
    else
      begin
        BackAttr := paramstr(3);
        for Index := 1 to length(BackAttr) do
          BackAttr[Index] := DnCase(BackAttr[Index]);
        if not BackColorOk then
          DefaultBack
      end;
    if paramcount < 2 then
      DefaultText
    else
      begin
        TextAttr := paramstr(2);
        for Index := 1 to length(TextAttr) do
          TextAttr[Index] := DnCase(TextAttr[Index]);
        if not TextColorOk then
          DefaultText
      end;
    if paramcount = 0 then
      begin
        write('  Enter name of file to be converted: ');
        EnterFileName(InFileName, 3,6);
        if InFileName = '' then
          halt
      end
    else
      InFileName := paramstr(1);
    NameOutFile(InFileName, OutFileName)
  end;


  procedure ConvertAscii;              (* Convert ASCII text to new ".TP"     *)
  var                                  (* file.                               *)
    LeadingBlanks : boolean;
    Y_Axis,
    X_Axis,
    ScreenNumber  : byte;
    TempString    : string[ScreenWidth];

  begin
    Y_Axis := 0;
    ScreenNumber := 0;
    assign(InFile, InFileName);        (* Assign file to file variable *)
    reset(InFile);                     (* Point to beginning of disk file *)
    assign(OutFile, OutFileName);
    rewrite(OutFile);
    writeln(OutFile, '');
    writeln(OutFile, 'program ', FirstPartFN, ';');
    writeln(OutFile, '');
    writeln(OutFile, 'uses');
    writeln(OutFile, '  crt;');
    writeln(OutFile, '');
    writeln(OutFile, 'BEGIN');
    writeln(OutFile, '  clrscr;');
    writeln(OutFile, '  textcolor(', TextAttr, ');');
    writeln(OutFile, '  textbackground(', BackAttr, ');');
    repeat
      LeadingBlanks := true;
      X_Axis := 0;
      TempString := '';
      readln(InFile, InString);        (* Read in first line of disk file *)
      if length(InString) = 0 then
        inc(Y_Axis)
      else
        begin
          for Index := 1 to length(InString) do
            begin
              if (InString[Index] = ' ') and LeadingBlanks = true then
                inc(X_Axis)
              else
                begin
                  LeadingBlanks := false;
                  TempString := concat(TempString + InString[Index])
                end;
              if InString[Index] = '''' then (* Single quotation mark *)
                TempString := concat(TempString + '''')
            end;
          inc(Y_Axis);
          inc(X_Axis);
          if Y_Axis > ScreenLength then
            begin
              Y_Axis := 1;
              inc(ScreenNumber);
              writeln(OutFile, '  readln;');
              writeln(OutFile, '  clrscr;');
              writeln(OutFile, '');
              writeln(OutFile, '  (* Screen ', ScreenNumber:3, ' *)');
              writeln(OutFile, '')
            end;
          writeln(OutFile, '  gotoxy(', X_Axis:2, ',', Y_Axis:2, ');');
          writeln(OutFile, '  writeln(''', TempString, ''');')
        end;
    until eof(InFile);
    writeln(OutFile, '  normvideo');
    writeln(OutFile, 'END.');
    close(InFile);
    close(OutFile)
  end;

BEGIN                                  (* Main Program *)
  CheckColor(ColorMode);
  clrscr;
  ProcessParams;
  DisplaySampleText;
  ConvertAscii;
  writeln('  Conversion complete!');
END.

